@@ -29,7 +29,8 @@ test_that("get_klass returns error when date in wrong format", {
   )
 
   expect_error(
-    x <- get_klass(2, date = c("2024-01-01", "2023-01-01", "2022-01-01"),
+    x <- get_klass(2,
+      date = c("2024-01-01", "2023-01-01", "2022-01-01"),
       "You have provided too many dates."
     )
   )
@@ -53,7 +54,8 @@ test_that("get_klass returns classifications with future date", {
 
 
 test_that("get_klass returns correct output level", {
-  class_data <- get_klass(7, date = "2024-01-01",
+  class_data <- get_klass(7,
+    date = "2024-01-01",
     output_level = 1
   )
   expect_equal(nrow(class_data), 10)
@@ -62,7 +64,8 @@ test_that("get_klass returns correct output level", {
 
 
 test_that("get_klass returns english language", {
-  class_data <- get_klass(2, date = "2024-01-01",
+  class_data <- get_klass(2,
+    date = "2024-01-01",
     language = "en"
   )
   expect_equal(class_data$name, c("Male", "Female"))
@@ -70,7 +73,8 @@ test_that("get_klass returns english language", {
 
 
 test_that("get_klass returns a correspondence table", {
-  class_data <- get_klass(104, correspond = 131,
+  class_data <- get_klass(104,
+    correspond = 131,
     date = "2015-01-01"
   )
   expect_equal(class_data$sourceCode[1], "01")
@@ -78,10 +82,12 @@ test_that("get_klass returns a correspondence table", {
 
 
 test_that("get_klass returns a correspondence table in both directions", {
-  class_data1 <- get_klass(104, correspond = 131,
+  class_data1 <- get_klass(104,
+    correspond = 131,
     date = "2020-02-01"
   )
-  class_data2 <- get_klass(131, correspond = 104,
+  class_data2 <- get_klass(131,
+    correspond = 104,
     date = "2020-02-01"
   )
   expect_equal(class_data1$sourceCode[1], "03")
